
ALBUMART_EXTENSIONS = (
    'jpg',
    'jpeg',
    'png',
    'gif',
)


ALBUMART_FILENAMES = (
    'artwork.jpg',
    'artwork.jpeg',
    'artwork.png',
    'artwork.gif',
    'albumart.jpg',
    'albumart.jpeg',
    'albumart.png',
    'albumart.gif',
    'coverart.jpg',
    'cover.jpg',
    'cover.jpeg',
    'cover.png',
    'cover.gif',
    'album.jpg',
    'album.jpeg',
    'album.png',
    'album.gif',
    'front.jpg',
    'front.jpeg',
    'front.png',
    'front.gif',
    'back.jpg',
    'back.jpeg',
    'back.png',
    'back.gif',
)
