
# Tags text supported by all codecs
COMMON_TEXT_TAGS = (
    'album_artist',
    'artist',
    'composer',
    'conductor',
    'orchestra',
    'performer',
    'album',
    'title',
    'genre',
    'comment',
    'note',
    'description',
    'label',
    'copyright',
    'sort_artist',
    'sort_album',
    'sort_title',
)
