
TAG_FIELDS = {
    'album_artist':         ['Album Artist'],
    'artist':               ['Artist'],
    'arranger':             ['Arranger'],
    'author':               ['Author'],
    'composer':             ['Composer'],
    'conductor':            ['Conductor'],
    'ensemble':             ['Ensemble'],
    'orchestra':            ['Orchestra'],
    'performer':            ['Performer'],
    'publisher':            ['Publisher'],
    'lyricist':             ['Lyricist'],
    'album':                ['Album'],
    'title':                ['Title'],
    'part_number':          ['Part number'],
    'part':                 ['Part'],
    'genre':                ['Genre'],
    'comment':              ['Comment'],
    'note':                 ['Note'],
    'description':          ['Description'],
    'location':             ['Record Location'],
    'year':                 ['Year'],
    'recording_date':       ['Record Date'],
    'bpm':                  ['BPM'],
    'rating':               ['Rating'],
    'label':                ['Label'],
    'label_code':           ['LC'],
    'opus':                 ['Opus'],
    'catalog_number':       ['Catalog'],
    'isbn':                 ['ISBN'],
    'isrc':                 ['ISRC'],
    'ean':                  ['EAN/UPC'],
    'lyrics':               ['Lyrics'],
    'abstract':             ['Abstract'],
    'bibliography':         ['Bibliography'],
    'copyright':            ['Copyright'],
    'publication_right':    ['Publicationright'],
    'language':             ['Language'],
    'related':              ['Related'],
    'sourcemedia':          ['Media'],
    'sort_album_artist':    ['Sort Album Artist'],
    'sort_artist':          ['Sort Artist'],
    'sort_composer':        ['Sort Composer'],
    'sort_performer':       ['Sort Performer'],
    'sort_show':            ['Sort Show'],
    'sort_album':           ['Sort Album'],
    'sort_title':           ['Sort Title'],
    'track_number':         ['Track'],
    'total_tracks':         ['Track'],
    'disk_number':          ['Disk'],
    'total_disks':          ['Disk'],
    'album_art':            [''],
}

LIST_FIELDS = (
    'artist',
    'composer',
    'comment',
    'copyright',
    'genre',
    'language',
    'location',
    'publication_right',
    'related',
)

INTERNAL_FIELDS = {
    'album_gain':           ['REPLAYGAIN_ALBUM_GAIN'],
    'album_peak':           ['REPLAYGAIN_ALBUM_PEAK'],
    'track_gain':           ['REPLAYGAIN_TRACK_GAIN'],
    'track_peak':           ['REPLAYGAIN_TRACK_PEAK'],
}

INTEGER_FIELDS = (
    'track_number',
    'total_tracks',
    'disk_number',
    'total_disks',
)

FLOAT_FIELDS = (
    'bpm',
)

BOOLEAN_FIELDS = ()
